@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright © Thomm.o 2021
+# Copyright © tandemdude 2020-present
 #
 # This file is part of Lightbulb.
 #
@@ -25,6 +25,7 @@ import functools
 import importlib
 import inspect
 import logging
+import pathlib
 import re
 import sys
 import typing
@@ -33,6 +34,7 @@ import hikari
 from hikari.internal import ux
 from multidict import CIMultiDict
 
+from lightbulb import checks
 from lightbulb import commands
 from lightbulb import context as context_
 from lightbulb import errors
@@ -103,7 +105,7 @@ def when_mentioned_or(
             bot = lightbulb.Bot(prefix=lightbulb.when_mentioned_or(get_prefix), ...)
     """
 
-    async def get_prefixes(bot, message):
+    async def get_prefixes(bot: Bot, message: hikari.Message) -> typing.Sequence[str]:
         mentions = [f"<@{bot.get_me().id}> ", f"<@!{bot.get_me().id}> "]
 
         if callable(prefix_provider):
@@ -120,6 +122,7 @@ def when_mentioned_or(
         elif prefixes is None:
             return mentions
         else:
+            prefixes = typing.cast(collections.AsyncIterable, prefixes)
             return mentions + [prefix async for prefix in prefixes]
 
     return get_prefixes
@@ -127,7 +130,7 @@ def when_mentioned_or(
 
 # Prefixes may be a string or iterable of strings. A string is a sequence of strings too by definition,
 # so this type hint _is_ correct.
-def _return_prefix(_: typing.Any, __: typing.Any, *, prefixes: typing.Iterable[str]) -> typing.Sequence[str]:
+def _return_prefix(_: Bot, __: hikari.Message, *, prefixes: typing.Iterable[str]) -> typing.Iterable[str]:
     return prefixes
 
 
@@ -166,6 +169,11 @@ class Bot(hikari.GatewayBot):
         slash_commands_only (:obj:`bool`): Whether or not the bot will only be using slash commands to interact
             with discord. Defaults to ``False``. If this is ``False`` and no prefix is provided then an error will
             be raised. If ``True``, then you do not need to provide a prefix.
+        default_enabled_guilds (Union[ :obj:`int`, Iterable[ :obj:`int` ] ]): The IDs of the guilds you want slash
+            commands to be enabled in by default. If this is an empty sequence or ``None``, commands will be global by
+            default, and you will need to manually specify the guild ID per command to change this. If you only want to
+            enable a single guild, you can pass the ID directly instead of a one-element sequence. Defaults to
+            ``None``.
         **kwargs: Other parameters passed to the :class:`hikari.impl.bot.GatewayBot` constructor.
     """
 
@@ -173,7 +181,7 @@ class Bot(hikari.GatewayBot):
         self,
         token: str,
         *,
-        prefix: prefix_T = None,
+        prefix: typing.Optional[prefix_T] = None,
         insensitive_commands: bool = False,
         ignore_bots: bool = True,
         owner_ids: typing.Iterable[int] = (),
@@ -181,7 +189,8 @@ class Bot(hikari.GatewayBot):
         delete_unbound_slash_commands: bool = True,
         recreate_changed_slash_commands: bool = True,
         slash_commands_only: bool = False,
-        **kwargs,
+        default_enabled_guilds: typing.Optional[typing.Union[int, typing.Iterable[int]]] = None,
+        **kwargs: hikari.UndefinedType,
     ) -> None:
         super().__init__(token, **kwargs)
 
@@ -191,7 +200,7 @@ class Bot(hikari.GatewayBot):
         if not slash_commands_only:
             self.subscribe(hikari.MessageCreateEvent, self.handle)
         self.subscribe(hikari.InteractionCreateEvent, self.handle_slash_commands)
-        self.subscribe(hikari.StartingEvent, self._manage_slash_commands)
+        self.subscribe(hikari.StartingEvent, self._manage_slash_commands_listener)
 
         if isinstance(prefix, str):
             self.get_prefix = functools.partial(_return_prefix, prefixes=[prefix])
@@ -213,7 +222,7 @@ class Bot(hikari.GatewayBot):
         )
         self.commands: typing.Set[commands.Command] = set()
         """A set containing all commands and groups registered to the bot."""
-        self._checks = []
+        self._checks: typing.List[checks.Check] = []
 
         self._delete_unbound_slash_commands = delete_unbound_slash_commands
         self._recreate_changed_slash_commands = recreate_changed_slash_commands
@@ -225,9 +234,16 @@ class Bot(hikari.GatewayBot):
         ] = set()
         """A set containing all slash commands registered to the bot."""
 
+        if isinstance(default_enabled_guilds, int):
+            default_enabled_guilds = (default_enabled_guilds,)
+        self.default_enabled_guilds: typing.Iterable[int] = default_enabled_guilds or []
+        """An iterable of the enabled guilds to use by default."""
+
         self._app: typing.Optional[hikari.PartialApplication] = None
 
         self._help_impl = help_class(self)
+        # The extension currently being loaded/unloaded.
+        self._current_extension: typing.Optional[_ExtensionType] = None
 
     @staticmethod
     def print_banner(banner: typing.Optional[str], allow_color: bool, force_color: bool) -> None:
@@ -235,6 +251,11 @@ class Bot(hikari.GatewayBot):
         if banner == "hikari":
             sys.stdout.write("Thank you for using lightbulb!\n")
 
+    @property
+    def app(self) -> typing.Optional[hikari.PartialApplication]:
+        """The application instance for the bot. Will be ``None`` unless the bot is running."""
+        return self._app
+
     @property
     def help_command(self) -> help_.HelpCommand:
         """The instance of the help class used by the bot."""
@@ -441,7 +462,9 @@ class Bot(hikari.GatewayBot):
         _LOGGER.debug("new group registered %r", func.name)
         return self._commands[func.name]
 
-    def add_check(self, func: typing.Callable[[context_.Context], typing.Coroutine[None, None, bool]]) -> None:
+    def add_check(
+        self, func: typing.Union[typing.Callable[[context_.Context], typing.Coroutine[None, None, bool]], checks.Check]
+    ) -> None:
         """
         Add a coroutine as a global check for the bot.
 
@@ -454,6 +477,8 @@ class Bot(hikari.GatewayBot):
         Returns:
             ``None``
         """
+        if not isinstance(func, checks.Check):
+            func = checks.Check(func)
         self._checks.append(func)
 
     def add_plugin(self, plugin: typing.Union[plugins.Plugin, typing.Type[plugins.Plugin]]) -> None:
@@ -600,6 +625,7 @@ class Bot(hikari.GatewayBot):
         Raises:
             :obj:`~.errors.ExtensionAlreadyLoaded`: If the extension has already been loaded.
             :obj:`~.errors.ExtensionMissingLoad`: If the extension to be loaded does not contain a ``load`` function.
+            :obj:`~.errors.ExtensionNotFound`: If the extension to be loaded does not exist.
 
         Example:
             This method is useful when wanting to split your bot up into multiple files.
@@ -617,17 +643,62 @@ class Bot(hikari.GatewayBot):
                     bot.add_plugin(MyPlugin())
         """
         if extension in self.extensions:
-            raise errors.ExtensionAlreadyLoaded(text=f"Extension {extension!r} is already loaded.")
+            raise errors.ExtensionAlreadyLoaded(f"Extension {extension!r} is already loaded.")
+
+        try:
+            module = importlib.import_module(extension)
+        except ModuleNotFoundError:
+            raise errors.ExtensionNotFound(f"No extension by the name {extension!r} was found") from None
 
-        module = importlib.import_module(extension)
         module = typing.cast(_ExtensionType, module)
+        self._current_extension = module
 
         if not hasattr(module, "load"):
-            raise errors.ExtensionMissingLoad(text=f"Extension {extension!r} is missing a load function")
+            raise errors.ExtensionMissingLoad(f"Extension {extension!r} is missing a load function")
         else:
             module.load(self)
             self.extensions.append(extension)
             _LOGGER.debug("new extension loaded %r", extension)
+        self._current_extension = None
+
+    def load_extensions_from(
+        self, path: typing.Union[str, pathlib.Path], *, recursive: bool = False, must_exist: bool = False
+    ) -> None:
+        """
+        Load all external extensions from a given directory. Files that begin with an underscore ( _ ) are ignored.
+        Every extension **must** contain a function ``load`` which takes a single argument which will be the bot
+        instance you are loading the extension into.
+
+        Args:
+            path (Union[ :obj:`str`, :obj:`pathlib.Path` ]): The directory to load extensions from.
+
+        Keyword Args:
+            recursive (:obj:`bool`): Whether to search the directory recursively. Defaults to False.
+            must_exist (:obj:`bool`): Whether the directory must exist before extensions can be loaded. If this is
+                False and the directory does not exist, no extensions will be loaded. If this is True, a
+                :obj:`FileNotFoundError` is thrown if the directory does not exist. Defaults to False.
+
+        Returns:
+            ``None``
+
+        Raises:
+            :obj:`~.errors.ExtensionAlreadyLoaded`: If the extension has already been loaded.
+            :obj:`~.errors.ExtensionMissingLoad`: If the extension to be loaded does not contain a ``load`` function.
+            :obj:`~.errors.ExtensionNotFound`: If the extension to be loaded does not exist.
+            :obj:`FileNotFoundError`: If the directory to load extensions from does not exist and ``must_exist``
+                is True.
+        """
+        if isinstance(path, str):
+            path = pathlib.Path(path)
+
+        if not path.is_dir():
+            if must_exist:
+                raise FileNotFoundError(f"{path} is not an existing directory")
+
+            return
+
+        for ext in path.glob(("**/" if recursive else "") + "[!_]*.py"):
+            self.load_extension(".".join([*ext.parts[:-1], ext.stem]))
 
     def unload_extension(self, extension: str) -> None:
         """
@@ -644,6 +715,7 @@ class Bot(hikari.GatewayBot):
         Raises:
             :obj:`~.errors.ExtensionNotLoaded`: If the extension has not been loaded.
             :obj:`~.errors.ExtensionMissingUnload`: If the extension does not contain an ``unload`` function.
+            :obj:`~.errors.ExtensionNotFound`: If the extension to be unloaded does not exist.
 
         Example:
 
@@ -661,18 +733,60 @@ class Bot(hikari.GatewayBot):
                     bot.remove_plugin("MyPlugin")
         """
         if extension not in self.extensions:
-            raise errors.ExtensionNotLoaded(text=f"Extension {extension!r} is not loaded.")
+            raise errors.ExtensionNotLoaded(f"Extension {extension!r} is not loaded.")
+
+        try:
+            module = importlib.import_module(extension)
+        except ModuleNotFoundError:
+            raise errors.ExtensionNotFound(f"No extension by the name {extension!r} was found") from None
 
-        module = importlib.import_module(extension)
         module = typing.cast(_ExtensionType, module)
+        self._current_extension = module
 
         if not hasattr(module, "unload"):
-            raise errors.ExtensionMissingUnload(text=f"Extension {extension!r} is missing an unload function")
+            raise errors.ExtensionMissingUnload(f"Extension {extension!r} is missing an unload function")
         else:
             module.unload(self)
             self.extensions.remove(extension)
             del sys.modules[extension]
             _LOGGER.debug("extension unloaded %r", extension)
+        self._current_extension = None
+
+    def unload_extensions(self, *extensions: str) -> None:
+        """
+        Unload multiple external extensions from the bot. This method relies on a function,
+        ``unload`` existing in the extensions to be unloaded which the bot will use to remove all
+        commands and/or plugins from the bot.
+
+        Args:
+            *extensions (:obj:`*str`): The names of the extensions to unload.
+
+        Returns:
+            ``None``
+
+        Raises:
+            :obj:`~.errors.ExtensionNotLoaded`: If the extension has not been loaded.
+            :obj:`~.errors.ExtensionMissingUnload`: If the extension does not contain an ``unload`` function.
+            :obj:`~.errors.ExtensionNotFound`: If the extension to be unloaded does not exist.
+        """
+        for ext in extensions:
+            self.unload_extension(ext)
+
+    def unload_all_extensions(self) -> None:
+        """
+        Unload all external extensions from the bot. This method relies on a function,
+        ``unload`` existing in all extensions which the bot will use to remove all
+        commands and/or plugins from the bot.
+
+        Returns:
+            ``None``
+
+        Raises:
+            :obj:`~.errors.ExtensionNotLoaded`: If the extension has not been loaded.
+            :obj:`~.errors.ExtensionMissingUnload`: If the extension does not contain an ``unload`` function.
+        """
+        for ext in self.extensions:
+            self.unload_extension(ext)
 
     def reload_extension(self, extension: str) -> None:
         """
@@ -698,6 +812,33 @@ class Bot(hikari.GatewayBot):
         else:
             del old
 
+    def reload_extensions(self, *extensions: str) -> None:
+        """
+        Reload multiple bot extensions. This method is atomic and so the bot will
+        revert to the previous loaded state if the extensions encounter problems
+        during unloading or loading.
+
+        Args:
+            *extensions (:obj:`*str`): The name of the extension to be reloaded.
+
+        Returns:
+            ``None``
+        """
+        for ext in extensions:
+            self.reload_extension(ext)
+
+    def reload_all_extensions(self) -> None:
+        """
+        Reload all bot extensions. This method is atomic and so the bot will
+        revert to the previous loaded state if the extensions encounter problems
+        during unloading or loading.
+
+        Returns:
+            ``None``
+        """
+        for ext in self.extensions:
+            self.reload_extension(ext)
+
     def walk_commands(self) -> typing.Generator[commands.Command, None, None]:
         """
         A generator that walks through all commands and subcommands registered to the bot.
@@ -1025,13 +1166,72 @@ class Bot(hikari.GatewayBot):
 
         await self.process_commands_for_event(event)
 
-    def add_slash_command(self, command: typing.Type[slash_commands.BaseSlashCommand], create: bool = False) -> None:
+    def _get_slash_commands_in_current_extension(
+        self,
+    ) -> typing.List[
+        typing.Union[typing.Type[slash_commands.SlashCommand], typing.Type[slash_commands.SlashCommandGroup]]
+    ]:
+        cmds = []
+        for item in dir(self._current_extension):
+            obj = getattr(self._current_extension, item)
+            if inspect.isclass(obj) and issubclass(
+                obj, (slash_commands.SlashCommand, slash_commands.SlashCommandGroup)
+            ):
+                cmds.append(obj)
+        return cmds
+
+    def autodiscover_slash_commands(self, create: bool = False) -> None:
+        """
+        Automatically discovers all slash command classes in the current extension
+        and adds each of them to the bot.
+
+        This should **only** be used in an extension's ``load`` function.
+
+        Args:
+            create (:obj:`bool`): Whether or not to send a create request to discord when the command is added.
+                Defaults to ``False``.
+
+        Returns:
+            ``None``
+        """
+        if self._current_extension is None:
+            return
+
+        for cmd in self._get_slash_commands_in_current_extension():
+            self.add_slash_command(cmd, create=create)
+
+    def autoremove_slash_commands(self, delete: bool = False) -> None:
+        """
+        Automatically discovers all slash command classes in the current extension
+        and removes each of them from the bot.
+
+        This should **only** be used in an extension's ``unload`` function.
+
+        Args:
+            delete (:obj:`bool`): Whether or not to delete the command from discord when it is removed.
+                Defaults to ``False``.
+
+        Returns:
+            ``None``
+        """
+        if self._current_extension is None:
+            return
+
+        for cmd in self._get_slash_commands_in_current_extension():
+            cmd = cmd(self)
+            self.remove_slash_command(cmd.name, delete=delete)
+
+    def add_slash_command(
+        self,
+        command: typing.Union[typing.Type[slash_commands.SlashCommand], typing.Type[slash_commands.SlashCommandGroup]],
+        create: bool = False,
+    ) -> None:
         """
         Registers a slash command with the bot.
 
         Args:
-            command (Type[:obj:`~lightbulb.slash_commands.SlashCommandBase`]): The slash command class to register
-                to the bot. This should **not** be instantiated.
+            command (Union[Type[:obj:`~lightbulb.slash_commands.SlashCommand`], Type[:obj:`~lightbulb.slash_commands.SlashCommandGroup`]]): The
+                slash command class to register to the bot. This should **not** be an instance.
             create (:obj:`bool`): Whether or not to send a create request to discord when the command is added.
                 Defaults to ``False``.
 
@@ -1157,7 +1357,7 @@ class Bot(hikari.GatewayBot):
         try:
             await command(context)
         except Exception as ex:
-            if not isinstance(ex, errors.CheckFailure):
+            if not isinstance(ex, (errors.CheckFailure, errors.CommandIsOnCooldown)):
                 ex = errors.CommandInvocationError(f"{type(ex).__name__}: {ex}", ex)
 
             error_event = events.SlashCommandErrorEvent(app=self, exception=ex, context=context, command=command)
@@ -1172,7 +1372,7 @@ class Bot(hikari.GatewayBot):
         else:
             await self.dispatch(events.SlashCommandCompletionEvent(app=self, command=command, context=context))
 
-    async def _manage_slash_commands(self, _):
+    async def _manage_slash_commands(self):
         self._app = await self.rest.fetch_application()
 
         global_slash_commands = {c.name: c for c in await self.rest.fetch_application_commands(self._app)}
@@ -1195,15 +1395,15 @@ class Bot(hikari.GatewayBot):
         # purging every command.
         remaining_globals = {} if self._delete_unbound_slash_commands is True else global_slash_commands
         if self._delete_unbound_slash_commands:
-            _LOGGER.debug("purging unbound slash commands")
+            _LOGGER.info("purging unbound slash commands")
             for cmd in global_slash_commands.values():
                 # if an implementation of the slash command is not found
                 if self._slash_commands.get(cmd.name) is None:
-                    _LOGGER.debug("deleting global slash command %r", cmd.name)
+                    _LOGGER.info("deleting global slash command %r", cmd.name)
                     await cmd.delete()
                 # if our implementation of the slash command is specific to guilds
                 elif self._slash_commands[cmd.name].enabled_guilds:
-                    _LOGGER.debug("deleting global slash command %r", cmd.name)
+                    _LOGGER.info("deleting global slash command %r", cmd.name)
                     await cmd.delete()
                 else:
                     remaining_globals[cmd.name] = cmd
@@ -1214,11 +1414,11 @@ class Bot(hikari.GatewayBot):
                 for cmd, guild_id in cmds:
                     # if an implementation of the slash command is not found
                     if self._slash_commands.get(cmd.name) is None:
-                        _LOGGER.debug("deleting slash command %r from guild %r", cmd.name, str(cmd.guild_id))
+                        _LOGGER.info("deleting slash command %r from guild %r", cmd.name, str(cmd.guild_id))
                         await cmd.delete()
                     # if our implementation of the slash command doesn't contain an entry for this guild
-                    elif cmd.guild_id not in self._slash_commands[cmd.name].enabled_guilds:
-                        _LOGGER.debug("deleting slash command %r from guild %r", cmd.name, str(cmd.guild_id))
+                    elif cmd.guild_id not in (self._slash_commands[cmd.name].enabled_guilds or []):
+                        _LOGGER.info("deleting slash command %r from guild %r", cmd.name, str(cmd.guild_id))
                         await cmd.delete()
                     else:
                         # We are assuming here that all the guild commands with the same name have the same
@@ -1238,7 +1438,7 @@ class Bot(hikari.GatewayBot):
         def compare_commands(
             lb_cmd: typing.Union[slash_commands.SlashCommand, slash_commands.SlashCommandGroup],
             hk_cmd: hikari.Command,
-            _guild_ids: typing.Optional[typing.List] = None,
+            _guild_ids: typing.Optional[typing.List[hikari.Snowflakeish]] = None,
         ) -> bool:
             # If one command is global and the other isn't
             if (not lb_cmd.enabled_guilds and _guild_ids) or (lb_cmd.enabled_guilds and not _guild_ids):
@@ -1264,10 +1464,10 @@ class Bot(hikari.GatewayBot):
                     continue
 
                 if not compare_commands(self._slash_commands[cmd.name], cmd):
-                    _LOGGER.debug("recreating global slash command %r as it appears to have changed", cmd.name)
+                    _LOGGER.info("recreating global slash command %r as it appears to have changed", cmd.name)
                     await self._slash_commands[cmd.name].auto_create(self._app)
                 else:
-                    _LOGGER.debug(
+                    _LOGGER.info(
                         "not recreating global slash command %r as it doesn't appear to have changed", cmd.name
                     )
 
@@ -1276,12 +1476,10 @@ class Bot(hikari.GatewayBot):
                     continue
 
                 if not compare_commands(self._slash_commands[cmd.name], cmd, guild_ids):
-                    _LOGGER.debug("recreating guild slash command %r as it appears to have changed", cmd.name)
+                    _LOGGER.info("recreating guild slash command %r as it appears to have changed", cmd.name)
                     await self._slash_commands[cmd.name].auto_create(self._app)
                 else:
-                    _LOGGER.debug(
-                        "not recreating guild slash command %r as it doesn't appear to have changed", cmd.name
-                    )
+                    _LOGGER.info("not recreating guild slash command %r as it doesn't appear to have changed", cmd.name)
 
         all_cmd_names = [
             *[c.name for c in remaining_globals.values()],
@@ -1289,5 +1487,14 @@ class Bot(hikari.GatewayBot):
         ]
         for cmd_name, cmd in self._slash_commands.items():
             if cmd_name not in all_cmd_names:
-                _LOGGER.debug("creating slash command %r as it does not seem to exist yet", cmd_name)
+                _LOGGER.info("creating slash command %r as it does not seem to exist yet", cmd_name)
                 await cmd.auto_create(self._app)
+
+    async def _manage_slash_commands_listener(self, _) -> None:
+        try:
+            await self._manage_slash_commands()
+        except hikari.ForbiddenError as exc:
+            _LOGGER.error(
+                "Could not complete slash command setup. Was your bot invited with the application.commands scope?",
+            )
+            raise exc
