@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright © Thomm.o 2021
+# Copyright © tandemdude 2020-present
 #
 # This file is part of Lightbulb.
 #
@@ -23,8 +23,10 @@ __all__: typing.Final[typing.List[str]] = [
     "Group",
     "command",
     "group",
+    "typing_override",
 ]
 
+import datetime
 import functools
 import inspect
 import logging
@@ -48,6 +50,7 @@ from lightbulb.converters import _UnionConverter
 if typing.TYPE_CHECKING:
     from lightbulb import checks as checks_
     from lightbulb import plugins
+    from lightbulb.checks import T_predicate
 
 _LOGGER = logging.getLogger("lightbulb")
 
@@ -68,6 +71,7 @@ _CONVERTER_CLASS_MAPPING = {
     hikari.Invite: converters.invite_converter,
     hikari.Colour: converters.colour_converter,
     hikari.Color: converters.colour_converter,
+    datetime.datetime: converters.timestamp_converter,
 }
 
 
@@ -131,11 +135,13 @@ class SignatureInspector:
         command (:obj:`~.commands.Command`): The command to inspect the arguments of.
     """
 
+    overriden_ns = {}
+
     def __init__(self, command: Command) -> None:
         self.command = command
         self.has_self = isinstance(command, _BoundCommandMarker)
         signature = inspect.signature(command._callback)
-        self.converters = self.parse_signature(signature)
+        self.converters = self.parse_signature(command._callback, signature)
         self.arguments = [p.name for p in signature.parameters.values()]
         self.arguments.pop(0)
         if self.has_self:
@@ -174,24 +180,30 @@ class SignatureInspector:
 
         return _Converter(annotation if annotation is not inspect.Parameter.empty else str)
 
-    def parse_signature(self, signature: inspect.Signature):
+    @staticmethod
+    def get_annotations(callback):
+        return typing.get_type_hints(callback, localns=SignatureInspector.overriden_ns)
+
+    def parse_signature(self, callback, signature: inspect.Signature):
         """
         Parse the command's callback signature into a list of the converters used
         for each command argument.
 
         Args:
+            callback: The command's callback
             signature (:obj:`inspect.Signature`): The signature of the command callback.
 
         Returns:
             List of converters for command arguments in the correct order.
         """
         arg_converters = []
+        all_annotations = self.get_annotations(callback)
 
         for idx, param in enumerate(signature.parameters.values()):
             if idx == 0 or (idx == 1 and self.has_self):
                 continue
 
-            converter: _BaseConverter = self.get_converter(param.annotation)
+            converter: _BaseConverter = self.get_converter(all_annotations.get(param.name, str))
 
             if param.kind is inspect.Parameter.KEYWORD_ONLY:
                 converter = _ConsumeRestConverter(converter, param.name)
@@ -227,7 +239,7 @@ class Command:
 
     def __init__(
         self,
-        callback: typing.Callable,
+        callback: typing.Callable[[context_.Context], typing.Coroutine[None, None, None]],
         name: str,
         allow_extra_arguments: bool,
         aliases: typing.Iterable[str],
@@ -240,7 +252,7 @@ class Command:
         self._aliases = aliases
         self.hidden = hidden
         self._checks: typing.List[checks_.Check] = []
-        self._check_exempt_predicate = lambda ctx: False
+        self._check_exempt_predicate: T_predicate = lambda ctx: False
         self._raw_error_listener = None
         self._raw_before_invoke = None
         self._raw_after_invoke = None
@@ -402,7 +414,7 @@ class Command:
 
             .. code-block:: python
 
-                @lightbulb.owner_only()
+                @lightbulb.check(lightbulb.owner_only)
                 @bot.command()
                 async def foo(ctx):
                     await ctx.respond("bar")
@@ -740,3 +752,22 @@ def group(**kwargs):
         )
 
     return decorate
+
+
+def typing_override(ns: typing.Dict[str, typing.Any]):
+    """
+    Add additional assistance to :obj:`typing.get_type_hints` for when command signatures cannot be
+    resolved correctly. This can be called as a function or used as a decorator.
+
+    Most users should never run into this problem.
+
+    Args:
+        ns (Dict[:obj:`str`, :obj:`typing.Any`): Dictionary of string annotation name/symbol to the object
+            that the annotation should be resolved into.
+    """
+    SignatureInspector.overriden_ns.update(ns)
+
+    def decorate(obj: T) -> T:
+        return obj
+
+    return decorate
