@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright © Thomm.o 2021
+# Copyright © tandemdude 2020-present
 #
 # This file is part of Lightbulb.
 #
@@ -126,22 +126,10 @@ class Context:
         """Optional timestamp of the previous edit of the context message."""
         return self._message.edited_timestamp
 
-    # XXX: these are at the time of writing typing.Sequence objects, but will
-    # be refactored to use typing.AbstractSet as part of https://github.com/nekokatt/hikari/issues/273.
     @property
-    def user_mentions(self) -> typing.Collection[hikari.Snowflake]:
-        """The users mentioned in the context message."""
-        return self._message.user_mentions
-
-    @property
-    def role_mentions(self) -> typing.Collection[hikari.Snowflake]:
-        """The roles mentioned in the context message."""
-        return self._message.role_mentions
-
-    @property
-    def channel_mentions(self) -> typing.Collection[hikari.Snowflake]:
-        """The channels mentioned in the context message."""
-        return self._message.channel_mentions
+    def mentions(self) -> hikari.Mentions:
+        """The mentions that exist in the message that triggered the command."""
+        return self._message.mentions
 
     @property
     def attachments(self) -> typing.Sequence[hikari.Attachment]:
@@ -160,31 +148,12 @@ class Context:
         returns the raw prefix.
         """
 
-        def replace(match):
+        def replace(match: re.Match) -> str:
             user = self.bot.cache.get_user(hikari.Snowflake(match.group(1)))
             return f"@{user}" if user is not None else self.prefix
 
         return re.sub(r"<@!?(\d+)>", replace, self.prefix)
 
-    @property
-    def guild(self) -> typing.Optional[hikari.Guild]:
-        """
-        The cached :obj:`hikari.Guild` instance for the context's guild ID.
-
-        This will be None if the bot is stateless, the guild is not found in the cache,
-        or the context is for a command run in DMs.
-        """
-
-        warnings.warn(
-            "The context property 'guild' is deprecated and scheduled for removal in version 1.4. "
-            "You should use 'get_guild()' instead.",
-            DeprecationWarning,
-        )
-
-        if self.guild_id is not None:
-            return self.bot.cache.get_available_guild(self.guild_id)
-        return None
-
     def get_guild(self) -> typing.Optional[hikari.Guild]:
         """
         The cached :obj:`hikari.Guild` instance for the context's guild ID.
@@ -196,25 +165,6 @@ class Context:
             return self.bot.cache.get_available_guild(self.guild_id)
         return None
 
-    @property
-    def channel(self) -> typing.Optional[hikari.TextableChannel]:
-        """
-        The cached :obj:`hikari.TextableChannel` instance for the context's channel ID.
-
-        This will be None if the bot is stateless, the channel is not found in the cache,
-        or the context is for a command run in DMs.
-        """
-
-        warnings.warn(
-            "The context property 'channel' is deprecated and scheduled for removal in version 1.4. "
-            "You should use 'get_channel()' instead.",
-            DeprecationWarning,
-        )
-
-        if self.guild_id is not None:
-            return self.bot.cache.get_guild_channel(self.channel_id)
-        return None
-
     def get_channel(self) -> typing.Optional[hikari.TextableChannel]:
         """
         The cached :obj:`hikari.TextableChannel` instance for the context's channel ID.
