@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright © Thomm.o 2021
+# Copyright © tandemdude 2020-present
 #
 # This file is part of Lightbulb.
 #
@@ -54,15 +54,12 @@ along with the converters they 'replace' can be seen below:
 - :obj:`hikari.Invite` (:obj:`~.invite_converter`)
 - :obj:`hikari.Colour` (:obj:`~.colour_converter`)
 - :obj:`hikari.Color` (:obj:`~.color_converter`)
+- :obj:`datetime.datetime` (:obj:`~.timestamp_converter`)
 
 
 .. warning:: For the supplied converters, some functionality will not be possible depending on the intents and/or
     cache settings of your bot application and object. If the bot does not have a cache then the converters can
     only work for arguments of ID or mention and **not** any form of name.
-
-.. warning:: If you use ``from __future__ import annotations`` then you **will not** be able to use converters
-    in your commands. Instead of converting the arguments, the raw, unconverted arguments will be passed back
-    to the command.
 """
 from __future__ import annotations
 
@@ -80,13 +77,14 @@ __all__: typing.Final[typing.List[str]] = [
     "invite_converter",
     "colour_converter",
     "color_converter",
+    "timestamp_converter",
     "Greedy",
 ]
 
 import collections
+import datetime
 import re
 import typing
-import warnings
 
 import hikari
 
@@ -102,6 +100,7 @@ USER_MENTION_REGEX: typing.Final[typing.Pattern] = re.compile(r"<@!?(\d+)>")
 CHANNEL_MENTION_REGEX: typing.Final[typing.Pattern] = re.compile(r"<#(\d+)>")
 ROLE_MENTION_REGEX: typing.Final[typing.Pattern] = re.compile(r"<@&(\d+)>")
 EMOJI_MENTION_REGEX: typing.Final[typing.Pattern] = re.compile(r"<a?:\w+:(\d+)>")
+TIMESTAMP_MENTION_REGEX: typing.Final[typing.Pattern] = re.compile(r"<t:(\d+)(?::[tTdDfFR])?>")
 
 
 class WrappedArg(collections.UserString):
@@ -264,16 +263,6 @@ async def guild_voice_channel_converter(arg: WrappedArg) -> hikari.GuildVoiceCha
     return channel
 
 
-# Deprecated, as this may change in the future to not just be a GuildVoiceChannel should Discord add new features,
-# so this shouldn't be used as it may cause breaking changes later.
-def voice_channel_converter(arg: WrappedArg) -> typing.Coroutine[typing.Any, typing.Any, hikari.GuildVoiceChannel]:
-    warnings.warn(
-        "voice_channel_converter is deprecated, use guild_voice_channel_converter instead",
-        category=DeprecationWarning,
-    )
-    return guild_voice_channel_converter(arg)
-
-
 async def category_converter(arg: WrappedArg) -> hikari.GuildCategory:
     """
     Converter to transform a command argument into a :obj:`~hikari.GuildCategory` object.
@@ -440,6 +429,25 @@ async def color_converter(arg: WrappedArg) -> hikari.Color:
     return await colour_converter(arg)
 
 
+async def timestamp_converter(arg: WrappedArg) -> datetime.datetime:
+    """
+    Converter to transform a command argument into a :obj:`datetime.datetime` object.
+
+    Args:
+        arg (:obj:`WrappedArg`): Argument to transform.
+
+    Returns:
+        :obj:`datetime.datetime`: The datetime object resolved from the argument.
+
+    Raises:
+        :obj:`~.errors.ConverterFailure`: If the argument could not be resolved into a datetime object.
+    """
+    timestamp = None
+    if match := TIMESTAMP_MENTION_REGEX.match(arg.data):
+        timestamp = match.group(1)
+    return datetime.datetime.fromtimestamp(int(timestamp), datetime.timezone.utc)
+
+
 class Greedy(typing.Generic[T]):
     """
     A special converter that greedily consumes arguments until it either runs out of arguments, or a parser error
